@@ -1,11 +1,11 @@
 # Copyright (C) 2003-2008 by Dr. Dieter Maurer, Illtalstr. 25, D-66571 Bubach, Germany
-#       $Id: sorting.py,v 1.4 2011-03-22 06:52:51 dieter Exp $
+#       $Id: sorting.py,v 1.5 2019/04/22 05:55:55 dieter Exp $
 '''Auxiliary sorting module'''
 
-from BTrees.IIBTree import difference, IISet, IITreeSet
+from BTrees.IIBTree import difference, IISet, IITreeSet, intersection
 from BTrees.OOBTree import OOBTree
 
-from AdvancedQuery import _notPassed, intersection
+from .eval import _notPassed
 
 def sort(seq, sortSpecs, withSortValue):
   '''sort 'IISet/IITreeSet' *seq* according to *sortSpec*.
@@ -34,7 +34,7 @@ class _SortAccess:
     if index >= self._len or index < 0: raise IndexError
     seq = self._seq
     while index >= len(seq):
-      s = self._iter.next()
+      s = next(self._iter)
       if isinstance(s, tuple):
         # with sort values
         sv, did = s
@@ -94,8 +94,9 @@ class IndexSorter(Sorter):
       if _load is None:
         # not an optimized index
         items = sortIndex.items()
-        
-        _load = lambda (x1, x2): x2
+        # Py2/3 compatibility
+        #_load = lambda (x1, x2): x2
+        _load = lambda x: x[1]
         if sortReverse: items.reverse()
       elif sortReverse:
         gRO = getattr(sortIndex, 'getReverseOrder', None)
@@ -107,9 +108,14 @@ class IndexSorter(Sorter):
         ids = intersection(seq, _load(i))
         if ids:
           handled.update(ids); hn += len(ids)
+          assert len(handled) == hn, \
+              "`%s` cannot be used as sort index as it indexes objects under different values" % sortIndex   
           yield i, ids
       if hn != len(seq): yield None, difference(seq, handled)
     else:
+      # workaround for `plone.folder.nogopip.GopipIndex`:
+      #  its `documentToKeyMap` looks for `rs` in the caller
+      rs = seq
       # result relatively small -- sort via result
       m = OOBTree()
       keyFor = getattr(sortIndex, 'keyForDocument', None)
@@ -132,7 +138,7 @@ class IndexSorter(Sorter):
       if noValue: yield None, noValue
 
 
-def normSortSpecs(specs, withSortValue, cat):
+def normSortSpecs(specs, withSortValue, cat, eval_params):
   '''normalize sort specs *specs* and *withSortValue*.
 
   *specs* is a sequence of sort specifications.
@@ -144,16 +150,17 @@ def normSortSpecs(specs, withSortValue, cat):
   '''
   l= []; withValue = False
   for s in specs:
-    if hasattr(s, '_prepare'): s = s._prepare(cat); withValue = True
+    if hasattr(s, '_prepare'):
+      s = s._prepare(cat, eval_params); withValue = True
     else:
-      if isinstance(s, tuple): si,so= s
-      else: si=s; so= 'asc'
-      i= cat.indexes[si]
+      if isinstance(s, tuple): si, so = s
+      else: si = s; so = 'asc'
+      i = cat._catalog.getIndex(si)
       # ensure, the index supports sorting
       if not hasattr(i,'documentToKeyMap'):
-        raise ValueError,'Index not adequate for sorting: %s' % si
+        raise ValueError('Index not adequate for sorting: %s' % si)
       # check whether we should reverse the order
-      so= so.lower()
+      so = so.lower()
       if so in ('asc', 'ascending'): sr= 0
       elif so in ('desc', 'descending', 'reverse'): sr= 1
       s = IndexSorter(i, sr)
