@@ -1,6 +1,6 @@
-# Copyright (C) 2004-2010 by Dr. Dieter Maurer, Illtalstr. 25, D-66571 Bubach, Germany
+# Copyright (C) 2004-2019 by Dr. Dieter Maurer, Illtalstr. 25, D-66571 Bubach, Germany
 # see "LICENSE.txt" for details
-#       $Id: testAdvancedQuery.py,v 1.2 2010-06-12 08:27:14 dieter Exp $
+#       $Id: testAdvancedQuery.py,v 1.3 2019/04/22 05:56:00 dieter Exp $
 
 # this no longer works for Zope 2.12 -- we drop this support
 ### as specified by "ZopeTestCase.framework"
@@ -11,9 +11,10 @@
 
 from Products.AdvancedQuery import *
 
-from TestBase import TestCase, getSuite
+from .TestBase import TestCase
 
-class TestAdvancedQuery(TestCase):
+
+class TestFieldIndex(TestCase):
 
   def testSlicing(self):
     c = self.catalog
@@ -48,7 +49,7 @@ class TestAdvancedQuery(TestCase):
     self._checkQuery(Generic('I1', {'query':'a'}), '13')
     self._checkQuery(Generic('I1', {'query':'a'}, filter=True), '13')
 
-  def testIndexes(self):
+  def testIndexed(self):
     self._checkQuery(Indexed('I1'), '12346')
 
   def testAnd(self):
@@ -66,6 +67,24 @@ class TestAdvancedQuery(TestCase):
     for i in range(10): q |= qb
     self._checkQuery(q, '13')
 
+  def testFilter(self):
+    self._checkQuery(Filter("I1", lambda iv: iv == "a"), "13")
+    self._checkQuery(Eq("I2", "A") & Filter("I1", lambda iv: iv == "a"), "1")
+
+  # `Not` tested in `testAnd`
+
+
+class TestKeywordIndex(TestFieldIndex):
+  _indexType = "KeywordIndex"
+  _multiValued = True
+
+  def testFilter(self):
+    self._checkQuery(Filter("I1", lambda iv: "a" in iv), "13")
+    self._checkQuery(Eq("I2", "A") & Filter("I1", lambda iv: "a" in iv), "1")
+
+
+    
+class TestSorting(TestCase):
   def testSortSmallIndex(self):
     C = self.catalog
     self._check(C.evalAdvancedQuery(And(), ('I1', ('I2','desc'))), '314265', False)
@@ -73,16 +92,13 @@ class TestAdvancedQuery(TestCase):
 
   def testSortLargeIndex(self):
     C = self.catalog; folder = self.folder
-    for i in range(7,100): self._addObject(folder, i, `i`, None)
+    for i in range(7,100): self._addObject(folder, i, repr(i), None)
     q = Eq('I2', 'A')
     self._check(C.evalAdvancedQuery(q, ('I1',)), '125', False)
     self._check(C.evalAdvancedQuery(q, (('I1','desc'),)), '215', False)
     
 
 
-class TestAdvancedQuery_ReverseOrder(TestAdvancedQuery):
-  _ReverseOrder = 1
-
 
 class TestIndexIndependant(TestCase):
   '''tests independent from index type.'''
@@ -142,14 +158,10 @@ class TestIndexIndependant(TestCase):
     rl = []; c = C._catalog
     for r in eval(all, (ranker(*rs), 'I1', 'I2')):
       rl.append(c.paths[r.data_record_id_])
-      rl.append(`r.data_record_score_[0]`)
+      rl.append(repr(int(r.data_record_score_[0])))
     self.assertEqual(''.join(rl), should)
 
 
-
-class TestAdvancedQuery_Field(TestAdvancedQuery):
-  _indexType = 'FieldIndex'
-
 class TestMatching(TestCase):
   _stringType = 1
 
@@ -158,14 +170,3 @@ class TestMatching(TestCase):
 
   def testRegexp(self):
     self._checkQuery(MatchRegexp('I1','.'), '12346')
-
-
-def test_suite(): return getSuite(TestAdvancedQuery,
-                                  TestAdvancedQuery_Field,
-                                  TestAdvancedQuery_ReverseOrder,
-                                  TestIndexIndependant,
-                                  TestMatching,
-                                  )
-
-# this no longer works for Zope 2.12 -- we drop this support
-##if __name__ == '__main__': framework()
